<?php

use Mufuphlex\Persistence\Repositories\RepositoryDoctrine;

class RepositoryDoctrineTest extends PHPUnit_Framework_TestCase
{
    private static $dummyModelName = '\Mufuphlex\Persistence\Models\DummyModel';

    public function tearDown()
    {
        parent::tearDown();
        Mockery::close();
    }

    public function testFactorySuccess()
    {
        $repo = static::getRepo();
        $this->assertInstanceOf('Mufuphlex\Persistence\Repositories\RepositoryDoctrine', $repo);
        $this->assertEquals(static::$dummyModelName, $repo->getModelClassName());
    }

    /**
     * @expectedException \InvalidArgumentException
     * @expectedExceptionMessage Option OPTION_MODEL_CLASS_NAME must be specified
     */
    public function testFactoryFailsOnNotPassedClassName()
    {
        $options = array();
        RepositoryDoctrine::factory($options);
    }

    /**
     * @expectedException \InvalidArgumentException
     * @expectedExceptionMessage Class "unexisting Xaba Xaba" does not exist
     */
    public function testFactoryFailsOnNonExistingClass()
    {
        $options = array(RepositoryDoctrine::OPTION_MODEL_CLASS_NAME => 'unexisting Xaba Xaba');
        RepositoryDoctrine::factory($options);
    }

    /**
     * @expectedException \LogicException
     * @expectedExceptionMessage modelClassName can not be reassigned
     */
    public function testSetModelClassNameFailsOnReassign()
    {
        $repo = new RepositoryDoctrine();
        $repo->setModelClassName(static::$dummyModelName);
        $repo->setModelClassName(static::$dummyModelName);
    }

    /**
     * @expectedException \LogicException
     * @expectedExceptionMessage $className must be a string, array given
     */
    public function testSetModelClassNameFailsOnNotString()
    {
        $repo = new RepositoryDoctrine();
        $repo->setModelClassName(array());
    }

    /**
     * @expectedException \LogicException
     * @expectedExceptionMessage entityManager can not be reassigned
     */
    public function testSetEntityManagerFailsOnRedefine()
    {
        $repo = static::getRepo();
        /** @var $entityManagerMock Doctrine\ORM\EntityManagerInterface */
        $entityManagerMock = Mockery::mock('Doctrine\ORM\EntityManagerInterface');
        $repo->setEntityManager($entityManagerMock);
    }

    public function testSaveSuccess()
    {
        $model = new \Mufuphlex\Persistence\Models\DummyModel();
        $repo = static::getRepoWithMock($model);
        $repo->save($model);
    }

    /**
     * @expectedException PHPUnit_Framework_Error
     */
    public function testSaveFailsOnNotInterfacePassed()
    {
        $model = new stdClass();
        /** @var $entityManagerMock Doctrine\ORM\EntityManagerInterface */
        $entityManagerMock = Mockery::mock('Doctrine\ORM\EntityManagerInterface');
        $entityManagerMock->shouldReceive('persist')->never();
        $entityManagerMock->shouldReceive('flush')->never();
        $repo = new RepositoryDoctrine();
        $repo->setEntityManager($entityManagerMock);
        $repo->save($model);
    }

    /**
     * @expectedException \InvalidArgumentException
     * @expectedExceptionMessage $model must implement ModelPersistentDoctrineInterface
     */
    public function testSaveFailsOnNotDoctrineInterfacePassed()
    {
        $model = new \Mufuphlex\Persistence\Models\DummyPersistentModel();
        /** @var $entityManagerMock Doctrine\ORM\EntityManagerInterface */
        $entityManagerMock = Mockery::mock('Doctrine\ORM\EntityManagerInterface');
        $entityManagerMock->shouldReceive('persist')->never();
        $entityManagerMock->shouldReceive('flush')->never();
        $repo = new RepositoryDoctrine();
        $repo->setEntityManager($entityManagerMock);
        $repo->save($model);
    }

    /**
     * @expectedException \LogicException
     * @expectedExceptionMessage entityManager is not set yet
     */
    public function testSaveFailsOnNotSetEntityManager()
    {
        $model = new \Mufuphlex\Persistence\Models\DummyModel();
        $repo = new RepositoryDoctrine();
        $repo->save($model);
    }

    public function testFindByIdNull()
    {
        $id = 1;
        $entityManagerMock = Mockery::mock('Doctrine\ORM\EntityManagerInterface');
        $entityManagerMock->shouldReceive('find')->withArgs(array(static::$dummyModelName, $id))->once();
        $repo = new RepositoryDoctrine();
        $repo->setEntityManager($entityManagerMock);
        $repo->setModelClassName(static::$dummyModelName);
        $model = $repo->findById($id);
        $this->assertNull($model);
    }

    /**
     * @expectedException \LogicException
     * @expectedExceptionMessage modelClassName is not set yet
     */
    public function testFindByIdFailsOnNotSetClassName()
    {
        $id = 1;
        $entityManagerMock = Mockery::mock('Doctrine\ORM\EntityManagerInterface');
        $entityManagerMock->shouldReceive('find')->never();
        $repo = new RepositoryDoctrine();
        $repo->setEntityManager($entityManagerMock);
        $repo->findById($id);
    }

    /**
     * @return RepositoryDoctrine
     */
    private static function getRepo()
    {
        $options = array(
            RepositoryDoctrine::OPTION_SETUP_PATHS => array(),
            RepositoryDoctrine::OPTION_CONNECTION_CONFIG => array(
                'driver' => 'pdo_sqlite',
                'path' => __DIR__ . '/db.sqlite'
            ),
            RepositoryDoctrine::OPTION_MODEL_CLASS_NAME => static::$dummyModelName
        );
        return RepositoryDoctrine::factory($options);
    }

    /**
     * @param $model
     * @return RepositoryDoctrine
     */
    private static function getRepoWithMock($model)
    {
        $entityManagerMock = static::getEntityManagerMock($model);
        $repo = new RepositoryDoctrine();
        $repo->setEntityManager($entityManagerMock);
        return $repo;
    }

    /**
     * @param $model
     * @return Doctrine\ORM\EntityManagerInterface
     */
    private static function getEntityManagerMock($model)
    {
        $entityManagerMock = Mockery::mock('Doctrine\ORM\EntityManagerInterface');
        $entityManagerMock->shouldReceive('persist')->withArgs(array($model))->once();
        $entityManagerMock->shouldReceive('flush')->withNoArgs()->once();
        return $entityManagerMock;
    }
}